# --
# Kernel/Modules/OcurrenceControl.pm - frontend module
#
# Copyright (C) 2015-2018 - SeTIC - UFSC - http://setic.ufsc.br/
# (Rodrigo Goncalves) (rodrigo.g@ufsc.br)
#
# Version 2018-01-31 - Adjustments for OTRS 6
#
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
# --
#
package Kernel::Modules::InPersonTicket;

use strict;
use warnings;
use Data::Dumper;

our @ObjectDependencies = (
	"Kernel::Output::HTML::Layout", "Kernel::System::Log",
	"Kernel::System::Web::Request", "Kernel::System::DB",
	"Kernel::Config",               "Kernel::System::CustomerUser",
	"Kernel::System::Service",      "Kernel::System::DynamicField::Backend",
	"Kernel::System::DynamicField", "Kernel::System::User"
);

sub new {
	my ( $Type, %Param ) = @_;

	# allocate new hash for object
	my $Self = {%Param};
	bless( $Self, $Type );

	return $Self;
}

sub Run {
	my ( $Self, %Param ) = @_;
	my %Data = ();

	my $ConfigObject   = $Kernel::OM->Get("Kernel::Config");
	my $languageObject = $Kernel::OM->Get("Kernel::Language");
	my $layoutObject   = $Kernel::OM->Get("Kernel::Output::HTML::Layout");
	my $paramObject    = $Kernel::OM->Get("Kernel::System::Web::Request");
	my $dbObject       = $Kernel::OM->Get("Kernel::System::DB");
	my $serviceObject  = $Kernel::OM->Get("Kernel::System::Service");

	$Self->BuildServiceBlock();
	$Self->BuildDynamicFieldsFields();

	my $config = $ConfigObject->Get("Ticket::Frontend::InPersonTicket");
	$layoutObject->AddJSData(
		Key   => "InPersonTicket.ServiceRealized",
		Value => $config->{ServiceConfiguredMessage}
	);

	if ( $paramObject->GetParam( Param => "Subaction" )
		&& ( $paramObject->GetParam( Param => "Subaction" ) eq "GetUserName" ) )
	{
		my $content = $Self->GetUserName(
			UserID => $paramObject->GetParam( Param => "UserID" ) );

		return $layoutObject->Attachment(
			ContentType => 'application/text; charset='
			  . $layoutObject->{Charset},
			Content => $content,
			Type    => 'inline',
			NoCache => 1,
		);
	}
	elsif ( $paramObject->GetParam( Param => "Subaction" )
		&& ( $paramObject->GetParam( Param => "Subaction" ) eq "Add" ) )
	{
		return $Self->CreateTicket();
	}

	my $Output = $layoutObject->Header(
		Title => $languageObject->Translate("In person ticket") );
	$Output .= $layoutObject->NavigationBar();
	$Output .= $layoutObject->Output( TemplateFile => 'InPersonTicket' );
	
	# set up rich text editor
	my %ParamCK = ();
	$layoutObject->SetRichTextParameters(
    	Data => \%ParamCK,
	);

	$Output .= $layoutObject->Footer();
	return $Output;

	return $Output;

}

sub GetUserName {
	my ( $Self, %Param ) = @_;

	my $userObject = $Kernel::OM->Get("Kernel::System::CustomerUser");
	my %dados = $userObject->CustomerUserDataGet( User => $Param{"UserID"} );

	if (%dados) {
		return $dados{UserFirstname};
	}
	else {
		return "";
	}

}

sub BuildDynamicFieldsFields {
	my ( $Self, %Param ) = @_;

	my $DynamicFieldBackendObject =
	  $Kernel::OM->Get('Kernel::System::DynamicField::Backend');
	my $DynamicFieldObject = $Kernel::OM->Get("Kernel::System::DynamicField");
	my $ConfigObject       = $Kernel::OM->Get("Kernel::Config");
	my $config       = $ConfigObject->Get("Ticket::Frontend::InPersonTicket");
	my $layoutObject = $Kernel::OM->Get("Kernel::Output::HTML::Layout");
	my $paramObject  = $Kernel::OM->Get("Kernel::System::Web::Request");
	my $userObject   = $Kernel::OM->Get("Kernel::System::User");
	my @dfieldList =
	  @{ $DynamicFieldObject->DynamicFieldListGet( Valid => 1 ) };
	my @dynamicFieldsInuse =
	  split( ",", $config->{DynamicFields} || "unidade" );
	my %prefs = $userObject->GetPreferences( UserID => $Self->{UserID} );

	foreach (@dfieldList) {
		my $field = $_;
		if ( $field->{Name} ~~ @dynamicFieldsInuse
			|| '*' . $field->{Name} ~~ @dynamicFieldsInuse )
		{

			my $defValue = "";

			if ( $field->{Name} ~~ @dynamicFieldsInuse ) {
				$defValue = $prefs{ "IPT_DF_" . $field->{Name} };
			}

			my $DynamicFieldHTML = $DynamicFieldBackendObject->EditFieldRender(
				DynamicFieldConfig => $field,
				LayoutObject       => $layoutObject,
				ParamObject        => $paramObject,
				Value              => $defValue,
				Mandatory		   => 1,
				OverridePossibleNone => 1,
				UseDefaultValue => 0,
				Value => ""
			);

			$layoutObject->Block(
				Name => 'DynamicField',
				Data => {
					Name  => $field->{Name},
					Label => $DynamicFieldHTML->{Label},
					Field => $DynamicFieldHTML->{Field},
				},
			);
		}
	}

}

sub BuildServiceBlock {
	my ( $Self, %Param ) = @_;

	my $layoutObject    = $Kernel::OM->Get("Kernel::Output::HTML::Layout");
	my $paramObject     = $Kernel::OM->Get("Kernel::System::Web::Request");
	my $dbObject        = $Kernel::OM->Get("Kernel::System::DB");
	my $serviceObject   = $Kernel::OM->Get("Kernel::System::Service");

	my $services = $serviceObject->ServiceListGet( Valid => 1, UserID => 1 );
	my @services = @$services;
	@services = sort { $a->{Name} cmp $b->{Name} } @services;

	foreach my $service (@services) {
		$layoutObject->Block(
			Name => 'selectService',
			Data => {
				ServiceID   => $service->{ServiceID},
				ServiceName => $Self->GetServiceName(
					Services    => \@services,
					ServiceID   => $service->{ServiceID},
					ServiceName => $service->{Name}
				),
				Disabled => $Self->LeafService(
					Services  => \@services,
					ServiceID => $service->{ServiceID}
				) ? "" : "disabled"
			}
		);
	}
}

sub GetServiceName {
	my ( $Self, %Param ) = @_;

	if ( $Self->LeafService(%Param) ) {
		my @tokens = split( "::", $Param{ServiceName} );
		return "&nbsp;&nbsp;" x ( @tokens - 1 ) . @tokens[ @tokens - 1 ];
	}
	else {
		return $Param{ServiceName};
	}
}

sub LeafService {
	my ( $Self, %Param ) = @_;

	my $services  = $Param{Services};
	my @services  = @$services;
	my $serviceID = $Param{ServiceID};

	foreach my $service (@services) {
		if (   ( exists $service->{ParentID} )
			&& ( $service->{ParentID} == $serviceID ) )
		{
			return 0;
		}
	}

	return 1;
}

sub GetQueueID() {
	my ( $Self, %Param ) = @_;

	my $ParamObject        = $Kernel::OM->Get("Kernel::System::Web::Request");
	my $QueueObject        = $Kernel::OM->Get("Kernel::System::Queue");
	my $DynamicFieldObject = $Kernel::OM->Get("Kernel::System::DynamicField");
	my $ConfigObject       = $Kernel::OM->Get("Kernel::Config");
	my $LayoutObject       = $Kernel::OM->Get("Kernel::Output::HTML::Layout");
	my $DynamicFieldBackendObject =
	  $Kernel::OM->Get('Kernel::System::DynamicField::Backend');

	# Gets default Queue
	my $ConfigTicket =
	  $ConfigObject->Get("Ticket::Frontend::CustomerTicketMessage");

	my $QueueDefault = $ConfigTicket->{"QueueDefault"};

	my %QueueListID = $QueueObject->QueueList( Valid => 1 );
	my %QueueList = reverse $QueueObject->QueueList( Valid => 1 );
	my $QueueID   = $QueueList{$QueueDefault};
	my $Queue     = $QueueListID{$QueueID};

	$Self->{ConfigModule} =
	  $ConfigObject->Get("Ticket::Frontend::InPersonTicket");

	# Checks if there is a field for the queue
	my $QueuePrefix = $Self->{ConfigModule}->{"QueuePrefix"};
	my $QueueField  = $Self->{ConfigModule}->{"QueueField"};

	if ( $QueuePrefix && $QueueField ) {
		my $dynFieldQueue =
		  $DynamicFieldObject->DynamicFieldGet( Name => $QueueField );
		my $QueueSelected = $DynamicFieldBackendObject->EditFieldValueGet(
			DynamicFieldConfig => $dynFieldQueue,
			ParamObject        => $ParamObject,
			LayoutObject       => $LayoutObject,
		);

		my $QueueName = "$QueuePrefix $QueueSelected";

		if ( $QueueList{$QueueName} ) {
			$QueueID = $QueueList{$QueueName};
			$Queue   = $QueueName;
		}
	}

	return ( $Queue, $QueueID );
}

sub CreateTicket {
	my ( $Self, %Param ) = @_;
	my %Data = ();

	my $languageObject = $Kernel::OM->Get("Kernel::Language");
	my $TicketObject   = $Kernel::OM->Get("Kernel::System::Ticket");
	my $ConfigObject   = $Kernel::OM->Get("Kernel::Config");
	my $ParamObject    = $Kernel::OM->Get("Kernel::System::Web::Request");
	my $BackendObject =
	  $Kernel::OM->Get("Kernel::System::DynamicField::Backend");
	my $DynamicFieldObject = $Kernel::OM->Get("Kernel::System::DynamicField");
	my $CustomerUserObject = $Kernel::OM->Get("Kernel::System::CustomerUser");
	my $LayoutObject       = $Kernel::OM->Get("Kernel::Output::HTML::Layout");
	my $userObject         = $Kernel::OM->Get("Kernel::System::User");
	my $DynamicFieldBackendObject =
	  $Kernel::OM->Get('Kernel::System::DynamicField::Backend');

	# Queue
	my ( $Queue, $QueueID ) = $Self->GetQueueID(%Param);
	$Data{QueueID} = $QueueID;
	my $email = $ParamObject->GetParam( Param => "userID" );

	$Self->{Config} =
	  $ConfigObject->Get("Ticket::Frontend::CustomerTicketMessage");

	$Self->{ConfigModule} =
	  $ConfigObject->Get("Ticket::Frontend::InPersonTicket");

	my $TicketID = $TicketObject->TicketCreate(
		Title    => $languageObject->Translate("In person ticket"),
		QueueID  => $QueueID,
		Priority => $Self->{Config}->{PriorityDefault},
		Lock     => 'unlock',
		State    => (
			$ParamObject->GetParam( Param => "concluded" )
			? 'closed successful'
			: 'new'
		),
		ServiceID  => $ParamObject->GetParam( Param => "ServiceID" ),
		TypeID     => $Self->{ConfigModule}->{TypeID},
		CustomerID => $Self->GetUserIDFromEmail(
			EMail             => $email,
			DefaultUserDomain => $Self->{ConfigModule}->{DefaultUserDomain},
			DefaultCustomerID => $Self->{ConfigModule}->{DefaultCustomerID}
		),
		CustomerUser => $Self->GetUserIDFromEmail(
			EMail             => $email,
			DefaultUserDomain => $Self->{ConfigModule}->{DefaultUserDomain},
			DefaultCustomerID => $Self->{ConfigModule}->{DefaultCustomerID}
		),
		OwnerID => $Self->{ConfigModule}->{DefaultOwnerID},
		UserID  => $Self->{ConfigModule}->{DefaultOwnerID},
	);

	my @dfieldList =
	  @{ $DynamicFieldObject->DynamicFieldListGet( Valid => 1 ) };
	my @dynamicFieldsInuse =
	  split( ",", $Self->{ConfigModule}->{DynamicFields} );

	foreach (@dfieldList) {
		my $field = $_;

		if ( $field->{Name} ~~ @dynamicFieldsInuse
			|| '*' . $field->{Name} ~~ @dynamicFieldsInuse )
		{
			my $value = $DynamicFieldBackendObject->EditFieldValueGet(
				DynamicFieldConfig => $field,
				ParamObject        => $ParamObject,
				LayoutObject       => $LayoutObject,
			);

			if ( $field->{Name} ~~ @dynamicFieldsInuse ) {
				$userObject->SetPreferences(
					UserID => $Self->{UserID},
					Key    => "IPT_DF_" . $field->{Name},
					Value  => $value,
				);
			}

			if ($value) {
				$DynamicFieldBackendObject->ValueSet(
					DynamicFieldConfig => $field,
					ObjectID           => $TicketID,
					Value              => $value,
					UserID             => $Self->{UserID},
				);
			}
		}
	}

	my $MimeType = 'text/plain';

	# Create article
	my $FullName = $ParamObject->GetParam( Param => "UserName" );

	my $From      = "\"$FullName\" <$email>";
	
	my $ArticleBackendObject = $Kernel::OM->Get('Kernel::System::Ticket::Article')->BackendForChannel(ChannelName => 'Email');
	
	my $ArticleID = $ArticleBackendObject->ArticleCreate(
		TicketID         => $TicketID,
		ArticleType      => 'phone',
		SenderType       => 'customer',
		From             => $email,
		To               => $Queue,
		Subject          => $languageObject->Translate("Customer service"),
		Body             => $ParamObject->GetParam( Param => "Description" ),
		MimeType         => $MimeType,
		Charset          => $LayoutObject->{UserCharset},
		UserID           => $Self->{ConfigModule}->{DefaultOwnerID},
		HistoryType      => $Self->{Config}->{HistoryType},
		HistoryComment   => $Self->{Config}->{HistoryComment} || '%%',
		AutoResponseType => '',
		OrigHeader       => {
			From    => $From,
			To      => $Queue,
			Subject => $languageObject->Translate("Customer service"),
			Body    => $ParamObject->GetParam( Param => "Description" ),
		},
		Queue => $Queue,
		IsVisibleForCustomer => 1
	);

	my $ReplyUser =
	  $Kernel::OM->Get('Kernel::System::TemplateGenerator')->Template(
		TemplateID => (
			  $ParamObject->GetParam( Param => "concluded" )
			? $Self->{ConfigModule}->{ReplyModelIDClosed}
			: $Self->{ConfigModule}->{ReplyModelID}
		),
		TicketID => $TicketID,
		UserID   => $Self->{ConfigModule}->{DefaultOwnerID},
	  );

	my $corpo = $ParamObject->GetParam( Param => "Description" );
	$ReplyUser =~ s/CONTEUDO/$corpo/g;

	my %sysAddr = $Kernel::OM->Get('Kernel::System::SystemAddress')
	  ->SystemAddressGet( ID => 1 );

	my $FromSystem = $sysAddr{Name};

	$MimeType = 'text/html';

	$ArticleID = $ArticleBackendObject->ArticleSend(
		TicketID => $TicketID,
		ArticleType =>
		  'note-internal',    # email-external|email-internal|phone|fax|...
		SenderType => 'agent',       # agent|system|customer
		From       => $FromSystem,
		To         => $email,        # not required but useful
		Subject  => $languageObject->Translate("Customer service"),   # required
		Body     => $ReplyUser,                                       # required
		Charset  => $LayoutObject->{UserCharset},
		MimeType => $MimeType,
		Loop => 0,    # 1|0 used for bulk emails
		HistoryType =>
		  'AddNote',    # Move|AddNote|PriorityUpdate|WebRequestCustomer|...
		HistoryComment => 'Auto-reply',
		NoAgentNotify => 0,    # if you don't want to send agent notifications
		UserID => $Self->{ConfigModule}->{DefaultOwnerID},
		IsVisibleForCustomer => 1
	);

	$Data{TicketNumber} =
	  $TicketObject->TicketNumberLookup( TicketID => $TicketID );

	# build output
	my $Output = $LayoutObject->Header(
		Title => $languageObject->Translate("In person ticket") );
	my %BlockData = ();
	$BlockData{TicketNumber} =
	    '<a href="/otrs/index.pl?Action=AgentTicketZoom;TicketID='
	  . $TicketID . '">'
	  . $Data{TicketNumber} . '</a>';
	$LayoutObject->Block(
		Name => 'NewTicketCreated',
		Data => \%BlockData
	);
	
	# set up rich text editor
	my %ParamCK = ();
	$LayoutObject->SetRichTextParameters(
    	Data => \%ParamCK,
	);
	

	$Output .= $LayoutObject->NavigationBar();
	$Output .= $LayoutObject->Output(
		Data         => \%Data,
		TemplateFile => 'InPersonTicket',
	);
	$Output .= $LayoutObject->Footer();
	return $Output;
}

sub GetUserIDFromEmail {
	my ( $Self, %Param ) = @_;

	my $email      = $Param{EMail};
	my $result     = $Param{DefaultCustomerID};
	my $domainList = $Param{DefaultUserDomain};

	if ( index( $email, '@' ) == -1 ) {
		return $email;
	}

	my $CustomerUserObject = $Kernel::OM->Get("Kernel::System::CustomerUser");

	if ($domainList) {
		my @domains = split( ",", $domainList );

		for my $domain (@domains) {
			my @parts = split( "@", $email );
			if ( $parts[1] eq $domain ) {
				my $id   = $parts[0];
				my %List = $CustomerUserObject->CustomerSearch(
					UserLogin => $id,
					Valid     => 1,
				);
				if ( keys %List ) {
					$result = $id;
					return $result;
				}
			}
		}
	}

	return $result;
}

1;
